#include <ft/test.h>
#include <ft/all-tests.h>
#include <gint/timer.h>
#include <setjmp.h>

static int timer_id = -1;

int callback(ft_test *t, jmp_buf *jb)
{
	/* We need to stop the timer now, as jumping out of the interrupt
	   handler will restore SR which enables the interrupt again */
	ft_log(t, "Stopping timer!\n");
	timer_stop(timer_id);

	ft_log(t, "Long jumping from callback()!\n");
	longjmp((*jb), 73);

	return TIMER_STOP;
}

static void _ft_setjmp_interrupt(ft_test *t)
{
	jmp_buf jb;
	int rc = setjmp(jb);

	if(rc == 0) {
		timer_id = timer_configure(TIMER_TMU, 1000,
			GINT_CALL(callback, (void *)t, (void *)&jb));
		if(timer_id >= 0) {
			ft_log(t, "Starting timer!\n");
			timer_start(timer_id);
		}
		else {
			ft_log(t, "Cannot get timer: %d\n", timer_id);
		}
		callback(t, &jb);
	}
	else {
		ft_log(t, "Got setjmp rc = %d\n", rc);
		ft_assert(t, rc == 73);
	}
}

ft_test ft_setjmp_interrupt = {
	.name = "Jump out of interrupt",
	.function = _ft_setjmp_interrupt,
};
